# Copyright 2010-2021 The pygit2 contributors
#
# This file is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License, version 2,
# as published by the Free Software Foundation.
#
# In addition to the permissions in the GNU General Public License,
# the authors give you unlimited permission to link the compiled
# version of this file into combinations with other programs,
# and to distribute those combinations without any restriction
# coming from the use of this file.  (The General Public License
# restrictions do apply in other respects; for example, they cover
# modification of the file, and distribution when not linked into
# a combined executable.)
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, 51 Franklin Street, Fifth Floor,
# Boston, MA 02110-1301, USA.

"""Tests for merging and information about it."""

from pathlib import Path

import pygit2
import pytest


def test_cherrypick_none(mergerepo):
    with pytest.raises(TypeError): mergerepo.cherrypick(None)

def test_cherrypick_invalid_hex(mergerepo):
    branch_head_hex = '12345678'
    with pytest.raises(KeyError): mergerepo.cherrypick(branch_head_hex)

def test_cherrypick_already_something_in_index(mergerepo):
    branch_head_hex = '03490f16b15a09913edb3a067a3dc67fbb8d41f1'
    branch_oid = mergerepo.get(branch_head_hex).id
    with (Path(mergerepo.workdir) / 'inindex.txt').open('w') as f:
        f.write('new content')
    mergerepo.index.add('inindex.txt')
    with pytest.raises(pygit2.GitError): mergerepo.cherrypick(branch_oid)


def test_cherrypick_remove_conflicts(mergerepo):
    other_branch_tip = '1b2bae55ac95a4be3f8983b86cd579226d0eb247'
    mergerepo.cherrypick(other_branch_tip)
    idx = mergerepo.index
    conflicts = idx.conflicts
    assert conflicts is not None
    conflicts['.gitignore']
    del idx.conflicts['.gitignore']
    with pytest.raises(KeyError): conflicts.__getitem__('.gitignore')
    assert idx.conflicts is None



# Copyright 2010-2021 The pygit2 contributors
#
# This file is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License, version 2,
# as published by the Free Software Foundation.
#
# In addition to the permissions in the GNU General Public License,
# the authors give you unlimited permission to link the compiled
# version of this file into combinations with other programs,
# and to distribute those combinations without any restriction
# coming from the use of this file.  (The General Public License
# restrictions do apply in other respects; for example, they cover
# modification of the file, and distribution when not linked into
# a combined executable.)
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, 51 Franklin Street, Fifth Floor,
# Boston, MA 02110-1301, USA.

import operator

import pygit2
import pytest

from . import utils


TREE_SHA = '967fce8df97cc71722d3c2a5930ef3e6f1d27b12'
SUBTREE_SHA = '614fd9a3094bf618ea938fffc00e7d1a54f89ad0'

def assertTreeEntryEqual(entry, sha, name, filemode):
    assert entry.hex == sha
    assert entry.name == name
    assert entry.filemode == filemode
    assert entry.raw_name == name.encode('utf-8')


def test_read_tree(barerepo):
    tree = barerepo[TREE_SHA]
    with pytest.raises(TypeError): tree[()]
    with pytest.raises(TypeError):
        tree / 123
    utils.assertRaisesWithArg(KeyError, 'abcd', lambda: tree['abcd'])
    utils.assertRaisesWithArg(IndexError, -4, lambda: tree[-4])
    utils.assertRaisesWithArg(IndexError, 3, lambda: tree[3])
    utils.assertRaisesWithArg(KeyError, 'abcd', lambda: tree / 'abcd')

    assert 3 == len(tree)
    sha = '7f129fd57e31e935c6d60a0c794efe4e6927664b'
    assert 'a' in tree
    assertTreeEntryEqual(tree[0], sha, 'a', 0o0100644)
    assertTreeEntryEqual(tree[-3], sha, 'a', 0o0100644)
    assertTreeEntryEqual(tree['a'], sha, 'a', 0o0100644)
    assertTreeEntryEqual(tree / 'a', sha, 'a', 0o0100644)

    sha = '85f120ee4dac60d0719fd51731e4199aa5a37df6'
    assert 'b' in tree
    assertTreeEntryEqual(tree[1], sha, 'b', 0o0100644)
    assertTreeEntryEqual(tree[-2], sha, 'b', 0o0100644)
    assertTreeEntryEqual(tree['b'], sha, 'b', 0o0100644)
    assertTreeEntryEqual(tree / 'b', sha, 'b', 0o0100644)

    sha = '297efb891a47de80be0cfe9c639e4b8c9b450989'
    assertTreeEntryEqual(tree['c/d'], sha, 'd', 0o0100644)
    assertTreeEntryEqual(tree / 'c/d', sha, 'd', 0o0100644)
    assertTreeEntryEqual(tree / 'c' / 'd', sha, 'd', 0o0100644)
    assertTreeEntryEqual(tree['c']['d'], sha, 'd', 0o0100644)
    assertTreeEntryEqual((tree / 'c')['d'], sha, 'd', 0o0100644)
    utils.assertRaisesWithArg(KeyError, 'ab/cd', lambda: tree['ab/cd'])
    utils.assertRaisesWithArg(KeyError, 'ab/cd', lambda: tree / 'ab/cd')
    utils.assertRaisesWithArg(KeyError, 'ab', lambda: tree / 'c' / 'ab')
    with pytest.raises(TypeError):
        tree / 'a' / 'cd'

def test_equality(barerepo):
    tree_a = barerepo['18e2d2e9db075f9eb43bcb2daa65a2867d29a15e']
    tree_b = barerepo['2ad1d3456c5c4a1c9e40aeeddb9cd20b409623c8']

    assert tree_a['a'] != tree_b['a']
    assert tree_a['a'] != tree_b['b']
    assert tree_a['b'] == tree_b['b']

def test_sorting(barerepo):
    tree_a = barerepo['18e2d2e9db075f9eb43bcb2daa65a2867d29a15e']
    assert list(tree_a) == sorted(reversed(list(tree_a)), key=pygit2.tree_entry_key)
    assert list(tree_a) != reversed(list(tree_a))

def test_read_subtree(barerepo):
    tree = barerepo[TREE_SHA]

    subtree_entry = tree['c']
    assertTreeEntryEqual(subtree_entry, SUBTREE_SHA, 'c', 0o0040000)
    assert subtree_entry.type == pygit2.GIT_OBJ_TREE
    assert subtree_entry.type_str == 'tree'

    subtree_entry = tree / 'c'
    assertTreeEntryEqual(subtree_entry, SUBTREE_SHA, 'c', 0o0040000)
    assert subtree_entry.type == pygit2.GIT_OBJ_TREE
    assert subtree_entry.type_str == 'tree'

    subtree = barerepo[subtree_entry.id]
    assert 1 == len(subtree)
    sha = '297efb891a47de80be0cfe9c639e4b8c9b450989'
    assertTreeEntryEqual(subtree[0], sha, 'd', 0o0100644)

    subtree_entry = (tree / 'c')
    assert subtree_entry == barerepo[subtree_entry.id]

def test_new_tree(barerepo):
    repo = barerepo
    b0 = repo.create_blob('1')
    b1 = repo.create_blob('2')
    st = repo.TreeBuilder()
    st.insert('a', b0, 0o0100644)
    subtree = repo[st.write()]

    t = repo.TreeBuilder()
    t.insert('x', b0, 0o0100644)
    t.insert('y', b1, 0o0100755)
    t.insert('z', subtree.id, pygit2.GIT_FILEMODE_TREE)
    tree = repo[t.write()]

    for name, oid, cls, filemode, type, type_str in [
        ('x', b0, pygit2.Blob, pygit2.GIT_FILEMODE_BLOB, pygit2.GIT_OBJ_BLOB, 'blob'),
        ('y', b1, pygit2.Blob, pygit2.GIT_FILEMODE_BLOB_EXECUTABLE, pygit2.GIT_OBJ_BLOB, 'blob'),
        ('z', subtree.id, pygit2.Tree, pygit2.GIT_FILEMODE_TREE, pygit2.GIT_OBJ_TREE, 'tree')]:

        assert name in tree
        obj = tree[name]
        assert isinstance(obj, cls)
        assert obj.name == name
        assert obj.filemode == filemode
        assert obj.type == type
        assert obj.type_str == type_str
        assert repo[obj.id].id == oid
        assert obj == repo[obj.id]

        obj = tree / name
        assert isinstance(obj, cls)
        assert obj.name == name
        assert obj.filemode == filemode
        assert obj.type == type
        assert obj.type_str == type_str
        assert repo[obj.id].id == oid
        assert obj == repo[obj.id]

def test_modify_tree(barerepo):
    tree = barerepo[TREE_SHA]
    with pytest.raises(TypeError): operator.setitem('c', tree['a'])
    with pytest.raises(TypeError): operator.delitem('c')

def test_iterate_tree(barerepo):
    """
    Testing that we're able to iterate of a Tree object and that the
    resulting sha strings are consitent with the sha strings we could
    get with other Tree access methods.
    """
    tree = barerepo[TREE_SHA]
    for tree_entry in tree:
        assert tree_entry == tree[tree_entry.name]

def test_iterate_tree_nested(barerepo):
    """
    Testing that we're able to iterate of a Tree object and then iterate
    trees we receive as a result.
    """
    tree = barerepo[TREE_SHA]
    for tree_entry in tree:
        if isinstance(tree_entry, pygit2.Tree):
            for tree_entry2 in tree_entry:
                pass

def test_deep_contains(barerepo):
    tree = barerepo[TREE_SHA]
    assert 'a' in tree
    assert 'c' in tree
    assert 'c/d' in tree
    assert 'c/e' not in tree
    assert 'd' not in tree

    assert 'd' in tree['c']
    assert 'e' not in tree['c']



# Copyright 2010-2021 The pygit2 contributors
#
# This file is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License, version 2,
# as published by the Free Software Foundation.
#
# In addition to the permissions in the GNU General Public License,
# the authors give you unlimited permission to link the compiled
# version of this file into combinations with other programs,
# and to distribute those combinations without any restriction
# coming from the use of this file.  (The General Public License
# restrictions do apply in other respects; for example, they cover
# modification of the file, and distribution when not linked into
# a combined executable.)
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, 51 Franklin Street, Fifth Floor,
# Boston, MA 02110-1301, USA.

from pathlib import Path

import pygit2
import pytest


@pytest.fixture
def repo(testrepopacked):
    testrepo = testrepopacked

    odb = pygit2.Odb()
    object_path = Path(testrepo.path) / 'objects'
    odb.add_backend(pygit2.OdbBackendPack(object_path), 1)
    odb.add_backend(pygit2.OdbBackendLoose(object_path, 0, False), 1)

    refdb = pygit2.Refdb.new(testrepo)
    refdb.set_backend(pygit2.RefdbFsBackend(testrepo))

    repo = pygit2.Repository()
    repo.set_odb(odb)
    repo.set_refdb(refdb)
    yield repo

def test_references(repo):
    refs = [(ref.name, ref.target.hex) for ref in repo.references.objects]
    assert sorted(refs) == [
        ('refs/heads/i18n', '5470a671a80ac3789f1a6a8cefbcf43ce7af0563'),
        ('refs/heads/master', '2be5719152d4f82c7302b1c0932d8e5f0a4a0e98')]

def test_objects(repo):
    a = repo.read('323fae03f4606ea9991df8befbb2fca795e648fa')
    assert (pygit2.GIT_OBJ_BLOB, b'foobar\n') == a



from pathlib import Path
import platform

import pytest
import pygit2
from . import utils


@pytest.fixture(scope='session', autouse=True)
def global_git_config():
    # Do not use global config for better test reproducibility.
    # https://github.com/libgit2/pygit2/issues/989
    levels = [pygit2.GIT_CONFIG_LEVEL_GLOBAL,
              pygit2.GIT_CONFIG_LEVEL_XDG,
              pygit2.GIT_CONFIG_LEVEL_SYSTEM]
    for level in levels:
        pygit2.settings.search_path[level] = ""

    # Fix tests running in AppVeyor
    if platform.system() == 'Windows':
        pygit2.option(pygit2.GIT_OPT_SET_OWNER_VALIDATION, 0)


@pytest.fixture
def pygit2_empty_key():
    path = Path(__file__).parent / 'keys' / 'pygit2_empty'
    return path, f'{path}.pub', 'empty'


@pytest.fixture
def barerepo(tmp_path):
    with utils.TemporaryRepository('barerepo.zip', tmp_path) as path:
        yield pygit2.Repository(path)


@pytest.fixture
def barerepo_path(tmp_path):
    with utils.TemporaryRepository('barerepo.zip', tmp_path) as path:
        yield pygit2.Repository(path), path


@pytest.fixture
def blameflagsrepo(tmp_path):
    with utils.TemporaryRepository('blameflagsrepo.zip', tmp_path) as path:
        yield pygit2.Repository(path)


@pytest.fixture
def dirtyrepo(tmp_path):
    with utils.TemporaryRepository('dirtyrepo.zip', tmp_path) as path:
        yield pygit2.Repository(path)


@pytest.fixture
def emptyrepo(barerepo, tmp_path):
    with utils.TemporaryRepository('emptyrepo.zip', tmp_path) as path:
        repo = pygit2.Repository(path)
        repo.remotes.create('origin', barerepo.path)
        yield repo


@pytest.fixture
def encodingrepo(tmp_path):
    with utils.TemporaryRepository('encoding.zip', tmp_path) as path:
        yield pygit2.Repository(path)


@pytest.fixture
def mergerepo(tmp_path):
    with utils.TemporaryRepository('testrepoformerging.zip', tmp_path) as path:
        yield pygit2.Repository(path)


@pytest.fixture
def testrepo(tmp_path):
    with utils.TemporaryRepository('testrepo.zip', tmp_path) as path:
        yield pygit2.Repository(path)


@pytest.fixture
def testrepo_path(tmp_path):
    with utils.TemporaryRepository('testrepo.zip', tmp_path) as path:
        yield pygit2.Repository(path), path


@pytest.fixture
def testrepopacked(tmp_path):
    with utils.TemporaryRepository('testrepopacked.zip', tmp_path) as path:
        yield pygit2.Repository(path)


@pytest.fixture
def gpgsigned(tmp_path):
    with utils.TemporaryRepository('gpgsigned.zip', tmp_path) as path:
        yield pygit2.Repository(path)



# Copyright 2010-2021 The pygit2 contributors
#
# This file is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License, version 2,
# as published by the Free Software Foundation.
#
# In addition to the permissions in the GNU General Public License,
# the authors give you unlimited permission to link the compiled
# version of this file into combinations with other programs,
# and to distribute those combinations without any restriction
# coming from the use of this file.  (The General Public License
# restrictions do apply in other respects; for example, they cover
# modification of the file, and distribution when not linked into
# a combined executable.)
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, 51 Franklin Street, Fifth Floor,
# Boston, MA 02110-1301, USA.

"""Tests for Mailmap."""

from pygit2 import Mailmap


TEST_MAILMAP = """\
# Simple Comment line
<cto@company.xx>                       <cto@coompany.xx>
Some Dude <some@dude.xx>         nick1 <bugs@company.xx>
Other Author <other@author.xx>   nick2 <bugs@company.xx>
Other Author <other@author.xx>         <nick2@company.xx>
Phil Hill <phil@company.xx>  # Comment at end of line
<joseph@company.xx>             Joseph <bugs@company.xx>
Santa Claus <santa.claus@northpole.xx> <me@company.xx>
"""

TEST_ENTRIES = [
    (None, "cto@company.xx", None, "cto@coompany.xx"),
    ("Some Dude", "some@dude.xx", "nick1", "bugs@company.xx"),
    ("Other Author", "other@author.xx", "nick2", "bugs@company.xx"),
    ("Other Author", "other@author.xx", None, "nick2@company.xx"),
    ("Phil Hill", None, None, "phil@company.xx"),
    (None, "joseph@company.xx", "Joseph", "bugs@company.xx"),
    ("Santa Claus", "santa.claus@northpole.xx", None, "me@company.xx")
]

TEST_RESOLVE = [
    ("Brad", "cto@company.xx", "Brad", "cto@coompany.xx"),
    ("Brad L", "cto@company.xx", "Brad L", "cto@coompany.xx"),
    ("Some Dude", "some@dude.xx", "nick1", "bugs@company.xx"),
    ("Other Author", "other@author.xx", "nick2", "bugs@company.xx"),
    ("nick3", "bugs@company.xx", "nick3", "bugs@company.xx"),
    ("Other Author", "other@author.xx", "Some Garbage", "nick2@company.xx"),
    ("Phil Hill", "phil@company.xx", "unknown", "phil@company.xx"),
    ("Joseph", "joseph@company.xx", "Joseph", "bugs@company.xx"),
    ("Santa Claus", "santa.claus@northpole.xx", "Clause", "me@company.xx"),
    ("Charles", "charles@charles.xx", "Charles", "charles@charles.xx")
]


def test_empty():
    mailmap = Mailmap()

    for (_, _, name, email) in TEST_RESOLVE:
        assert mailmap.resolve(name, email) == (name, email)


def test_new():
    mailmap = Mailmap()

    # Add entries to the mailmap
    for entry in TEST_ENTRIES:
        mailmap.add_entry(*entry)

    for (real_name, real_email, name, email) in TEST_RESOLVE:
        assert mailmap.resolve(name, email) == (real_name, real_email)


def test_parsed():
    mailmap = Mailmap.from_buffer(TEST_MAILMAP)

    for (real_name, real_email, name, email) in TEST_RESOLVE:
        assert mailmap.resolve(name, email) == (real_name, real_email)


# TODO: Add a testcase which uses .mailmap in a repo



# Copyright 2010-2022 The pygit2 contributors
#
# This file is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License, version 2,
# as published by the Free Software Foundation.
#
# In addition to the permissions in the GNU General Public License,
# the authors give you unlimited permission to link the compiled
# version of this file into combinations with other programs,
# and to distribute those combinations without any restriction
# coming from the use of this file.  (The General Public License
# restrictions do apply in other respects; for example, they cover
# modification of the file, and distribution when not linked into
# a combined executable.)
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, 51 Franklin Street, Fifth Floor,
# Boston, MA 02110-1301, USA.

from pygit2 import GIT_OBJ_COMMIT, Oid, Signature

content = """\
tree 4b825dc642cb6eb9a060e54bf8d69288fbee4904
parent 8496071c1b46c854b31185ea97743be6a8774479
author Ben Burkert <ben@benburkert.com> 1358451456 -0800
committer Ben Burkert <ben@benburkert.com> 1358451456 -0800

a simple commit which works\
"""

gpgsig = """\
-----BEGIN PGP SIGNATURE-----
Version: GnuPG v1.4.12 (Darwin)

iQIcBAABAgAGBQJQ+FMIAAoJEH+LfPdZDSs1e3EQAJMjhqjWF+WkGLHju7pTw2al
o6IoMAhv0Z/LHlWhzBd9e7JeCnanRt12bAU7yvYp9+Z+z+dbwqLwDoFp8LVuigl8
JGLcnwiUW3rSvhjdCp9irdb4+bhKUnKUzSdsR2CK4/hC0N2i/HOvMYX+BRsvqweq
AsAkA6dAWh+gAfedrBUkCTGhlNYoetjdakWqlGL1TiKAefEZrtA1TpPkGn92vbLq
SphFRUY9hVn1ZBWrT3hEpvAIcZag3rTOiRVT1X1flj8B2vGCEr3RrcwOIZikpdaW
who/X3xh/DGbI2RbuxmmJpxxP/8dsVchRJJzBwG+yhwU/iN3MlV2c5D69tls/Dok
6VbyU4lm/ae0y3yR83D9dUlkycOnmmlBAHKIZ9qUts9X7mWJf0+yy2QxJVpjaTGG
cmnQKKPeNIhGJk2ENnnnzjEve7L7YJQF6itbx5VCOcsGh3Ocb3YR7DMdWjt7f8pu
c6j+q1rP7EpE2afUN/geSlp5i3x8aXZPDj67jImbVCE/Q1X9voCtyzGJH7MXR0N9
ZpRF8yzveRfMH8bwAJjSOGAFF5XkcR/RNY95o+J+QcgBLdX48h+ZdNmUf6jqlu3J
7KmTXXQcOVpN6dD3CmRFsbjq+x6RHwa8u1iGn+oIkX908r97ckfB/kHKH7ZdXIJc
cpxtDQQMGYFpXK/71stq
=ozeK
-----END PGP SIGNATURE-----\
"""

gpgsig_content = """\
tree 4b825dc642cb6eb9a060e54bf8d69288fbee4904
parent 8496071c1b46c854b31185ea97743be6a8774479
author Ben Burkert <ben@benburkert.com> 1358451456 -0800
committer Ben Burkert <ben@benburkert.com> 1358451456 -0800
gpgsig -----BEGIN PGP SIGNATURE-----
 Version: GnuPG v1.4.12 (Darwin)
 
 iQIcBAABAgAGBQJQ+FMIAAoJEH+LfPdZDSs1e3EQAJMjhqjWF+WkGLHju7pTw2al
 o6IoMAhv0Z/LHlWhzBd9e7JeCnanRt12bAU7yvYp9+Z+z+dbwqLwDoFp8LVuigl8
 JGLcnwiUW3rSvhjdCp9irdb4+bhKUnKUzSdsR2CK4/hC0N2i/HOvMYX+BRsvqweq
 AsAkA6dAWh+gAfedrBUkCTGhlNYoetjdakWqlGL1TiKAefEZrtA1TpPkGn92vbLq
 SphFRUY9hVn1ZBWrT3hEpvAIcZag3rTOiRVT1X1flj8B2vGCEr3RrcwOIZikpdaW
 who/X3xh/DGbI2RbuxmmJpxxP/8dsVchRJJzBwG+yhwU/iN3MlV2c5D69tls/Dok
 6VbyU4lm/ae0y3yR83D9dUlkycOnmmlBAHKIZ9qUts9X7mWJf0+yy2QxJVpjaTGG
 cmnQKKPeNIhGJk2ENnnnzjEve7L7YJQF6itbx5VCOcsGh3Ocb3YR7DMdWjt7f8pu
 c6j+q1rP7EpE2afUN/geSlp5i3x8aXZPDj67jImbVCE/Q1X9voCtyzGJH7MXR0N9
 ZpRF8yzveRfMH8bwAJjSOGAFF5XkcR/RNY95o+J+QcgBLdX48h+ZdNmUf6jqlu3J
 7KmTXXQcOVpN6dD3CmRFsbjq+x6RHwa8u1iGn+oIkX908r97ckfB/kHKH7ZdXIJc
 cpxtDQQMGYFpXK/71stq
 =ozeK
 -----END PGP SIGNATURE-----

a simple commit which works\
"""
# NOTE: ^^^ mind the gap (space must exist after GnuPG header) ^^^
# XXX: seems macos wants the space while linux does not


def test_commit_signing(gpgsigned):
    repo = gpgsigned
    message = "a simple commit which works"
    author = Signature(
        name="Ben Burkert",
        email="ben@benburkert.com",
        time=1358451456,
        offset=-480,
    )
    committer = Signature(
        name="Ben Burkert",
        email="ben@benburkert.com",
        time=1358451456,
        offset=-480,
    )
    tree = "4b825dc642cb6eb9a060e54bf8d69288fbee4904"
    parents = ["8496071c1b46c854b31185ea97743be6a8774479"]

    # create commit string
    commit_string = repo.create_commit_string(
        author, committer, message, tree, parents
    )
    assert commit_string == content

    # create/retrieve signed commit
    oid = repo.create_commit_with_signature(content, gpgsig)
    commit = repo.get(oid)
    signature, payload = commit.gpg_signature

    # validate signed commit
    assert content == payload.decode("utf-8")
    assert gpgsig == signature.decode("utf-8")
    assert gpgsig_content == commit.read_raw().decode("utf-8")

    # perform sanity checks
    assert GIT_OBJ_COMMIT == commit.type
    assert "6569fdf71dbd99081891154641869c537784a3ba" == commit.hex
    assert commit.message_encoding is None
    assert message == commit.message
    assert 1358451456 == commit.commit_time
    assert committer == commit.committer
    assert author == commit.author
    assert tree == commit.tree.hex
    assert Oid(hex=tree) == commit.tree_id
    assert 1 == len(commit.parents)
    assert parents[0] == commit.parents[0].hex
    assert Oid(hex=parents[0]) == commit.parent_ids[0]


def test_get_gpg_signature_when_unsigned(gpgsigned):
    unhash = "5b5b025afb0b4c913b4c338a42934a3863bf3644"

    repo = gpgsigned
    commit = repo.get(unhash)
    signature, payload = commit.gpg_signature

    assert signature is None
    assert payload is None
